<?php

namespace App\Extended\Response;

use App\Constants\ErrorCode;
use App\Extended\Util\RequestDurationUtil;
use App\Extended\Util\RequestHostUtil;
use App\Extended\Util\RequestSessionUtil;
use Hyperf\Di\Annotation\Inject;
use Hyperf\HttpServer\Contract\RequestInterface;
use Hyperf\HttpServer\Contract\ResponseInterface;
use JsonSerializable;
use stdClass;

class Response {

    #[Inject]
    protected RequestInterface $request;

    #[Inject]
    protected ResponseInterface $response;

    private $data;
    private $code;
    private $message;
    private $auth;
    const KEY_AUTH = 'auth';
    const SUCCESS_MESSAGE = 'success';

    const KEY_CODE = 'code';
    const KEY_MESSAGE = 'message';
    const KEY_DATA = 'data';
    const KEY_SESSION_ID = 'session_id';
    const KEY_REQUEST_DURATION = 'duration';
    const KEY_PATCH = 'patch';
    const KEY_HOST_NAME = 'host_name';
    const KEY_ENV_NAME = 'env_name';

    public function __construct($data, int $code = 0, string $message = '') {
        $this->data = $data;
        $this->code = $code;
        $this->message = $message;
    }

    public static function make($data, int $code = 0, string $message = '') {
        return new self($data, $code, $message);
    }

    public function setAuth($auth): static {
        $this->auth = $auth;
        return $this;
    }

    public function response() {
        $responseData = $this->makeResponseData();
        return $this->response->json($responseData);
    }

    public function rpcResponse() {
        return $this->makeResponseData(true);
    }

    private function makeResponseData(bool $isRpc = false) {
        $json[self::KEY_SESSION_ID] = RequestSessionUtil::generate();
        $json[self::KEY_REQUEST_DURATION] = RequestDurationUtil::duration();
        $json[self::KEY_HOST_NAME] = RequestHostUtil::hostName();
        $json[self::KEY_ENV_NAME] = RequestHostUtil::envName();
        if (!$isRpc) {
            $json = [
                self::KEY_PATCH => $this->request->path(),
            ];
        }
        $json[self::KEY_CODE] = $this->code;
        if ($this->code == 0) {
            $json[self::KEY_MESSAGE] = self::SUCCESS_MESSAGE;
        } else {
            if (!empty($this->message)) {
                $json[self::KEY_MESSAGE] = $this->message;
            } else {
                $json[self::KEY_MESSAGE] = ErrorCode::getMessage($this->code);
            }
        }
        if (isset($this->auth)) {
            $json[self::KEY_AUTH] = $this->auth;
        }
        $data = $this->apiDataFilter($this->data);
        if (!is_null($data)) {
            $json[self::KEY_DATA] = $data;
        }

        return $json;
    }

    private function apiDataFilter($data) {

        if (is_string($data) || is_int($data) || is_float($data) || is_bool($data)) {
            return $data;
        } elseif (is_array($data)) {
            if (empty($data)) {
                return $data;
            }

            $returnArray = null;
            foreach ($data as $key => $value) {
                $tmp = $this->apiDataFilter($value);
                if (isset($tmp)) {
                    $returnArray[$key] = $tmp;
                }
            }
            return $returnArray;
        } elseif (is_object($data) && $data instanceof JsonSerializable) {
            $tmp = $data->jsonSerialize();
            if (is_array($tmp)) {
                return $this->apiDataFilter($tmp);
            }
        } elseif (is_object($data) && $data instanceof stdClass) {
            return $data;
        } elseif (is_object($data) && method_exists($data, 'toArray')) {
            $tmp = $data->toArray();
            if (is_array($tmp)) {
                return $this->apiDataFilter($tmp);
            }
        }

        return null;
    }
}